// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/src/core/utils/multi_color.dart';

/// {@template card_theme_extension}
/// Card theme extension that extends [ThemeExtension] and
/// implements copyWith, and lerp methods so you don't have to.
/// {@endtemplate}
class CardThemeExtension extends ThemeExtension<CardThemeExtension> {
  /// {@macro card_theme_extension}
  const CardThemeExtension({
    this.radius,
    this.padding,
    this.backgroundColors,
    this.borderColors,
    this.stroke,
    this.shadow,
    this.minSize,
    this.maxSize,
    this.titleStyle,
    this.subtitleStyle,
    this.bodyStyle,
  });

  /// Card radius
  ///
  /// Default to `BorderRadius.all(Radius.circular(12.0))`
  final BorderRadiusGeometry? radius;

  /// Padding and gaps of this card
  ///
  /// Default to `Theme.cardTheme.margin`
  final EdgeInsetsGeometry? padding;

  /// Card background gradient colors (from left to right)
  ///
  /// Default to `Theme.cardTheme.color`
  final MultiColor? backgroundColors;

  /// Border colors (from left to right).
  ///
  /// Default to `null`
  final MultiColor? borderColors;

  /// Stroke of the border
  ///
  /// Default to `null`
  final double? stroke;

  /// Drop shadow
  ///
  /// Default to `null`
  final BoxShadow? shadow;

  /// Minimum size of the card
  ///
  /// Default to `const Size(330, 0)`
  final Size? minSize;

  /// Maximum size of the card
  ///
  /// Default to `const Size(double.infinity, double.infinity)`
  final Size? maxSize;

  /// Title text style
  ///
  /// Default to `Theme.textTheme.titleLarge`
  final TextStyle? titleStyle;

  /// Subtitle text style
  ///
  /// Default to `Theme.textTheme.titleMedium`
  final TextStyle? subtitleStyle;

  /// Body text style
  ///
  /// Default to `Theme.textTheme.bodyMedium`
  final TextStyle? bodyStyle;

  @override
  ThemeExtension<CardThemeExtension> copyWith({
    BorderRadiusGeometry? radius,
    EdgeInsetsGeometry? padding,
    MultiColor? backgroundColors,
    MultiColor? borderColors,
    double? stroke,
    BoxShadow? shadow,
    Size? minSize,
    Size? maxSize,
    TextStyle? titleStyle,
    TextStyle? subtitleStyle,
    TextStyle? bodyStyle,
  }) =>
      CardThemeExtension(
        radius: radius ?? this.radius,
        padding: padding ?? this.padding,
        backgroundColors: backgroundColors ?? this.backgroundColors,
        borderColors: borderColors ?? this.borderColors,
        stroke: stroke ?? this.stroke,
        shadow: shadow ?? this.shadow,
        minSize: minSize ?? this.minSize,
        maxSize: maxSize ?? this.maxSize,
        titleStyle: titleStyle ?? this.titleStyle,
        subtitleStyle: subtitleStyle ?? this.subtitleStyle,
        bodyStyle: bodyStyle ?? this.bodyStyle,
      );

  @override
  ThemeExtension<CardThemeExtension> lerp(
    covariant ThemeExtension<CardThemeExtension>? other,
    double t,
  ) {
    if (other is! CardThemeExtension) {
      return this;
    }
    return CardThemeExtension(
      radius: BorderRadiusGeometry.lerp(radius, other.radius, t),
      padding: EdgeInsetsGeometry.lerp(padding, other.padding, t),
      backgroundColors:
          MultiColor.lerp(backgroundColors, other.backgroundColors, t),
      borderColors: MultiColor.lerp(borderColors, other.borderColors, t),
      shadow: BoxShadow.lerp(shadow, other.shadow, t),
      bodyStyle: TextStyle.lerp(bodyStyle, other.bodyStyle, t),
      titleStyle: TextStyle.lerp(titleStyle, other.titleStyle, t),
      subtitleStyle: TextStyle.lerp(subtitleStyle, other.subtitleStyle, t),
    );
  }
}
